==> kuklomenos-0.2.tar.gz: kuklomenos-0.2/main.cc <==
#include <cstdlib>
#include <cstdio>
#include <ctime>
#include <getopt.h>
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>

#include "state.h"
#include "geom.h"
#include "clock.h"
#include "data.h"
#include "settings.h"
#include "conffile.h"

SDL_Surface* screen = NULL;

enum EventsReturn
{
    ER_NONE,
    ER_QUIT,
    ER_RESTART
};

EventsReturn process_events(GameState& gameState, GameClock& gameClock)
{
    SDL_Event event;
    while (SDL_PollEvent(&event))
    {
	switch (event.type)
	{
	    case SDL_QUIT:
		return ER_QUIT;
	    case SDL_KEYDOWN:
		switch (event.key.keysym.sym)
		{
		    case SDLK_q:
			return ER_QUIT;
		    case SDLK_p:
			gameClock.paused = !gameClock.paused;
			break;
		    case SDLK_z:
			settings.zoomEnabled = !settings.zoomEnabled;
			break;
		    case SDLK_r:
			settings.rotatingView = !settings.rotatingView;
			break;
		    case SDLK_a:
			settings.useAA = !settings.useAA;
			break;
		    case SDLK_g:
			settings.showGrid = !settings.showGrid;
			break;
		    case SDLK_LEFTBRACKET:
			settings.fps = std::max(1, settings.fps-1);
			break;
		    case SDLK_RIGHTBRACKET:
			settings.fps = std::min(100, settings.fps+1);
			break;
		    case SDLK_MINUS:
		    case SDLK_KP_MINUS:
			gameClock.rate = 4*gameClock.rate/5;
			if (950 <= gameClock.rate && gameClock.rate <= 1050)
			    gameClock.rate = 1000;
			break;
		    case SDLK_PLUS:
		    case SDLK_KP_PLUS:
			gameClock.rate = std::max(5*gameClock.rate/4,
				gameClock.rate+1);
			if (950 <= gameClock.rate && gameClock.rate <= 1050)
			    gameClock.rate = 1000;
			break;
		    case SDLK_EQUALS:
			gameClock.rate = 1000;
			break;
		    case SDLK_SPACE:
		    case SDLK_RETURN:
			if (gameState.end)
			    return ER_RESTART;
			break;
		    default: ;
		}
		break;
	    case SDL_VIDEORESIZE:
		screenGeom = ScreenGeom(event.resize.w, event.resize.h);
		screen = SDL_SetVideoMode(event.resize.w, event.resize.h,
			screen->format->BitsPerPixel, screen->flags);
		break;
	    default: ;
	}
    }
    return ER_NONE;
}

void drawInfo(SDL_Surface* surface, GameState& gameState, GameClock& gameClock,
	float observedFPS)
{
    char fpsStr[5+8];
    snprintf(fpsStr, 5+8, "fps: %.1f/%d", observedFPS, settings.fps);

    char ratingStr[8+20+7];
    snprintf(ratingStr, 8+20+7, "rating: %s (%.1lf)",
	    ratingString((int)(gameState.rating)), gameState.rating);

    char rateStr[6+5];
    snprintf(rateStr, 6+5, "rate: %d.%d", gameClock.rate/1000,
	    (gameClock.rate%1000)/100);

    gfxPrimitivesSetFont(fontSmall,7,13);
    stringColor(surface, screenGeom.info.x, screenGeom.info.y, fpsStr, 0xffffffff);
    stringColor(surface, screenGeom.info.x, screenGeom.info.y+15*2, ratingStr, 0xffffffff);
    if (gameClock.rate != 1000)
	stringColor(surface, screenGeom.info.x, screenGeom.info.y+15*4, rateStr, 0xffffffff);
}

class OverlayText
{
    public:
	static const int text1Len = 20;
	static const int text2Len = 40;
	char text1[text1Len];
	char text2[text2Len];
	int colour;

	OverlayText();
	void blank();
	void draw(SDL_Surface* surface);
};

OverlayText::OverlayText() :
    colour(0)
{
    blank();
}
void OverlayText::blank()
{
    text1[0] = '\0';
    text2[0] = '\0';
}

void OverlayText::draw(SDL_Surface* surface)
{
    gfxPrimitivesSetFont(fontBig, 10, 20);
    stringColor(surface, screenGeom.centre.x - strlen(text1)*10/2, screenGeom.centre.y - 30,
	    text1, colour);
    stringColor(surface, screenGeom.centre.x - strlen(text2)*10/2, screenGeom.centre.y + 30,
	    text2, colour);
}

const int MIN_STEP = 30;

int main(int argc, char** argv)
{
    GameState gameState;
    GameClock gameClock;
    OverlayText overlayText;

    // Initialisation code adapted from that used in sdl_gfx,
    // (C) A. Schiffler 2001
    int w, h;
    int desired_bpp;
    Uint32 video_flags;

    /* Set default options and check command-line */
    w = 0;
    h = 0;
    desired_bpp = 0;
    video_flags = 0;

    while (1) {
	int c;
	static const struct option long_options[] =
	{
	    {"width", 1, 0, 'W'},
	    {"height", 1, 0, 'H'},
	    {"bpp", 1, 0, 'b'},
	    {"fps", 1, 0, 'f'},
	    {"rating", 1, 0, 'r'},
	    {"antialias", 0, 0, 'a'},
	    {"nogrid", 0, 0, 'G'},
	    {"nozoom", 0, 0, 'Z'},
	    {"norotate", 0, 0, 'R'},
	    {"fullscreen", 0, 0, 'F'},
	    {"resizable", 0, 0, 's'},
	    {"hwpalette", 0, 0, 'P'},
	    {"hwsurface", 0, 0, 'S'},
	    {"invulnerable", 0, 0, 'i'},
	    {"version", 0, 0, 'V'},
	    {"help", 0, 0, 'h'},
	    {0,0,0,0}
	};
	c = getopt_long(argc, argv, "W:H:b:f:r:I:aGZRFsPSiVh", long_options, NULL);
	if (c == -1)
	    break;
	switch (c)
	{
	    case 'W':
		w = atoi(optarg);
		break;
	    case 'H':
		h = atoi(optarg);
		break;
	    case 'b':
		desired_bpp = atoi(optarg);
		break;
	    case 'f':
		settings.fps = atoi(optarg);
		break;
	    case 'r':
		if (1.0 <= atof(optarg))
		    gameState.setRating(atof(optarg));
		else
		    printf("bad rating\n");
		break;
	    case 'F':
		video_flags |= SDL_FULLSCREEN;
		break;
	    case 's':
		video_flags |= SDL_RESIZABLE;
		break;
	    case 'P':
		video_flags |= SDL_HWPALETTE;
		break;
	    case 'S':
		video_flags |= SDL_HWSURFACE;
		break;
	    case 'Z':
		settings.zoomEnabled = false;
		break;
	    case 'R':
		settings.rotatingView = false;
		break;
	    case 'a':
		settings.useAA = true;
		break;
	    case 'G':
		settings.showGrid = false;
		break;
	    case 'i':
		settings.invuln = true;
		break;
	    case 'V':
		printf("%s\n", PACKAGE_STRING);
		exit(0);
	    case 'h':
		printf("Options:\n\t"
			"-W --width WIDTH\n\t-H --height HEIGHT\n\t-b --bpp BITS\n\t-f --fps FPS\n\t"
			"-F --fullscreen\n\t-S --resizable\n\t-P --hwpalette\n\t-S --hwsurface\n\t"
			"-Z --nozoom\n\t-R --norotate\n\t-G --nogrid\n\t-a --antialias\n\t"
			"-r --rating RATING\t\t1: harmless... 4: average... 9: elite\n\t"
			"-i --invulnerable\n\t-V --version\n\t-h --help\n");
		exit(0);
		break;

	    case '?':
		break;

	    default:
		printf("getopt returned unexpected character %c\n", c);
	}
    }

    if (gameState.rating == 0)
    {
	gameState.setRating(config.rating);
	config.shouldUpdateRating = true;
    }

    /* Force double buffering */
    video_flags |= SDL_DOUBLEBUF;

    /* Initialize SDL */
    if ( SDL_Init(SDL_INIT_EVERYTHING) < 0 ) {
	fprintf(stderr,
		"Couldn't initialize SDL: %s\n", SDL_GetError());
	exit(1);
    }
    atexit(SDL_Quit);			/* Clean up on exit */

    if (w == 0 || h == 0)
    {
	SDL_Rect** modes = SDL_ListModes(NULL, video_flags);
	if (modes == NULL)
	{
	    fprintf(stderr,
		    "SDL reports no modes available\n");
	    exit(1);
	}
	if (modes == (SDL_Rect**)(-1))
	{
	    // "All modes available"
	    w = 1024;
	    h = 768;
	}
	else
	{
	    // use the first (i.e. biggest) mode which is no bigger than
	    // 1024x768
	    SDL_Rect* bestMode;
	    do
	    {
		bestMode = *modes;
		if (bestMode->w <= 1024 && bestMode->h <= 768)
		    break;
	    } while (++modes);
	    w = bestMode->w;
	    h = bestMode->h;
	}
    }
	

    /* Initialize the display */
    screen = SDL_SetVideoMode(w, h, desired_bpp, video_flags);
    if ( screen == NULL ) {
	fprintf(stderr, "Couldn't set %dx%dx%d video mode: %s\n",
		w, h, desired_bpp, SDL_GetError());
	exit(1);
    }

    /* Show some info */
    printf("Set %dx%dx%d mode\n",
	    screen->w, screen->h, screen->format->BitsPerPixel);
    printf("Video surface located in %s memory.\n",
	    (screen->flags&SDL_HWSURFACE) ? "video" : "system");

    /* Check for double buffering */
    if ( screen->flags & SDL_DOUBLEBUF ) {
	printf("Double-buffering enabled - good!\n");
    }

    SDL_WM_SetCaption( "Kuklomenos", NULL );

    SDL_ShowCursor(SDL_DISABLE);

    srand(time(NULL));

    if ( !initFont() )
    {
	fprintf(stderr, "Failed to load font data file\n");
	exit(1);
    }

    screenGeom = ScreenGeom(w,h);

    // main loop
    Uint32 lastStateUpdate = 0;
    Uint32 ticksBefore, ticksAfter;
    Uint32 loopTicks = SDL_GetTicks();
    int timeTillNextFrame = 0;
    int delayTime, updateTime;
    float avFrameTime = 1000/settings.fps;
    const int avFrames = 10; // number of frames to average over
    EventsReturn eventsReturn = ER_NONE;
    bool quit = false;
    bool ended = false;
    while ( !quit ) {
	while (timeTillNextFrame > 0)
	{
	    delayTime = std::min(timeTillNextFrame, MIN_STEP);
	    SDL_Delay(delayTime);
	    timeTillNextFrame -= delayTime;

	    eventsReturn = process_events(gameState, gameClock);

	    switch (eventsReturn)
	    {
		case ER_RESTART:
		    {
			GameState newGameState;
			newGameState.setRating(gameState.rating);
			gameState = newGameState;
		    }
		    ended = false;
		    overlayText.blank();
		    break;
		case ER_QUIT:
		    quit = true;
		default: ;
	    }

	    updateTime = SDL_GetTicks() - lastStateUpdate;
	    lastStateUpdate += updateTime;
	    if (!gameClock.paused)
	    {
		gameState.update(gameClock.scale(updateTime));
		gameClock.update(updateTime);
	    }
	}

	ticksBefore = SDL_GetTicks();
	if (!gameClock.paused)
	{
	    gameState.draw(screen);
	    drawInfo(screen, gameState, gameClock, 1000.0/avFrameTime);
	    overlayText.draw(screen);
	    SDL_Flip(screen);
	}
	ticksAfter = SDL_GetTicks();

	const int renderingTicks = ticksAfter - ticksBefore;
	timeTillNextFrame =
	    std::max(1, (1000/settings.fps) - renderingTicks);

	if (!ended && gameState.end)
	{
	    ended = true;
	    switch (gameState.end)
	    {
		case END_DEAD:
		case END_EXTRACTED:
		    snprintf(overlayText.text1, overlayText.text1Len, "Failure.");
		    overlayText.colour = 0xff4444ff;
		    break;
		case END_WIN:
		    snprintf(overlayText.text1, overlayText.text1Len, "Victory!");
		    overlayText.colour = 0x88ff00ff;
		    break;
		default: ;
	    }

	    if (config.shouldUpdateRating && !settings.invuln)
	    {
		int old = (int)(gameState.rating);

		if (gameState.end == END_DEAD ||
			gameState.end == END_EXTRACTED)
		{
		    gameState.rating -= 0.1;
		    if (gameState.rating < 1)
			gameState.rating = 1.0;

		}
		else if (gameState.end == END_WIN)
		{
		    gameState.rating += 0.4;
		    if (gameState.rating > 9)
			gameState.rating = 9.0;

		}

		if ((int)(gameState.rating) != old)
		    snprintf(overlayText.text2, overlayText.text2Len, "New rating: \"%s\".",
			    ratingString( (int)(gameState.rating) ));
	    }
	}

	const int loopTime = SDL_GetTicks() - loopTicks;
	avFrameTime += (loopTime-avFrameTime)/avFrames;
	loopTicks = SDL_GetTicks();
    }
    
    if (gameState.extracted > 0)
    {
	// game is forfeit - reduce rating
	gameState.rating -= 0.1;
	if (gameState.rating < 1)
	    gameState.rating = 1.0;
    }
    if (config.shouldUpdateRating)
    {
	config.rating = gameState.rating;
	config.write();
    }
    SDL_Quit();
}

==> kuklomenos-0.2.1.tar.gz: kuklomenos-0.2.1/main.cc <==
/*
 * Kuklomenos
 * Copyright (C) 2008 Martin Bays <mbays@sdf.lonestar.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */

#include <cstdlib>
#include <cstdio>
#include <ctime>
#include <getopt.h>
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>

#include "state.h"
#include "geom.h"
#include "clock.h"
#include "data.h"
#include "settings.h"
#include "conffile.h"
#include "random.h"

SDL_Surface* screen = NULL;

enum EventsReturn
{
    ER_NONE,
    ER_QUIT,
    ER_RESTART
};

EventsReturn process_events(GameState& gameState, GameClock& gameClock)
{
    SDL_Event event;
    while (SDL_PollEvent(&event))
    {
	switch (event.type)
	{
	    case SDL_QUIT:
		return ER_QUIT;
	    case SDL_KEYDOWN:
		switch (event.key.keysym.sym)
		{
		    case SDLK_q:
			return ER_QUIT;
		    case SDLK_p:
			gameClock.paused = !gameClock.paused;
			break;
		    case SDLK_z:
			settings.zoomEnabled = !settings.zoomEnabled;
			break;
		    case SDLK_r:
			settings.rotatingView = !settings.rotatingView;
			break;
		    case SDLK_a:
			settings.useAA = !settings.useAA;
			break;
		    case SDLK_g:
			settings.showGrid = !settings.showGrid;
			break;
		    case SDLK_LEFTBRACKET:
			settings.fps = std::max(1, settings.fps-1);
			break;
		    case SDLK_RIGHTBRACKET:
			settings.fps = std::min(100, settings.fps+1);
			break;
		    case SDLK_MINUS:
		    case SDLK_KP_MINUS:
			gameClock.rate = 4*gameClock.rate/5;
			if (950 <= gameClock.rate && gameClock.rate <= 1050)
			    gameClock.rate = 1000;
			break;
		    case SDLK_PLUS:
		    case SDLK_KP_PLUS:
			gameClock.rate = std::max(5*gameClock.rate/4,
				gameClock.rate+1);
			if (950 <= gameClock.rate && gameClock.rate <= 1050)
			    gameClock.rate = 1000;
			break;
		    case SDLK_EQUALS:
			gameClock.rate = 1000;
			break;
		    case SDLK_SPACE:
		    case SDLK_RETURN:
			if (gameState.end)
			    return ER_RESTART;
			break;
		    default: ;
		}
		break;
	    case SDL_VIDEORESIZE:
		screenGeom = ScreenGeom(event.resize.w, event.resize.h);
		screen = SDL_SetVideoMode(event.resize.w, event.resize.h,
			screen->format->BitsPerPixel, screen->flags);
		break;
	    default: ;
	}
    }
    return ER_NONE;
}

void drawInfo(SDL_Surface* surface, GameState& gameState, GameClock& gameClock,
	float observedFPS)
{
    char fpsStr[5+8];
    snprintf(fpsStr, 5+8, "fps: %.1f/%d", observedFPS, settings.fps);

    char ratingStr[8+20+7];
    snprintf(ratingStr, 8+20+7, "rating: %s (%.1lf)",
	    ratingString((int)(gameState.rating)), gameState.rating);

    char rateStr[6+5];
    snprintf(rateStr, 6+5, "rate: %d.%d", gameClock.rate/1000,
	    (gameClock.rate%1000)/100);

    gfxPrimitivesSetFont(fontSmall,7,13);
    stringColor(surface, screenGeom.info.x, screenGeom.info.y, fpsStr, 0xffffffff);
    stringColor(surface, screenGeom.info.x, screenGeom.info.y+15*2, ratingStr, 0xffffffff);
    if (gameClock.rate != 1000)
	stringColor(surface, screenGeom.info.x, screenGeom.info.y+15*4, rateStr, 0xffffffff);
}

class OverlayText
{
    public:
	static const int text1Len = 20;
	static const int text2Len = 40;
	char text1[text1Len];
	char text2[text2Len];
	int colour;

	OverlayText();
	void blank();
	void draw(SDL_Surface* surface);
};

OverlayText::OverlayText() :
    colour(0)
{
    blank();
}
void OverlayText::blank()
{
    text1[0] = '\0';
    text2[0] = '\0';
}

void OverlayText::draw(SDL_Surface* surface)
{
    gfxPrimitivesSetFont(fontBig, 10, 20);
    stringColor(surface, screenGeom.centre.x - strlen(text1)*10/2, screenGeom.centre.y - 30,
	    text1, colour);
    stringColor(surface, screenGeom.centre.x - strlen(text2)*10/2, screenGeom.centre.y + 30,
	    text2, colour);
}

const int MIN_STEP = 30;

int main(int argc, char** argv)
{
    GameState gameState;
    GameClock gameClock;
    OverlayText overlayText;

    // Initialisation code adapted from that used in sdl_gfx,
    // (C) A. Schiffler 2001
    int w, h;
    int desired_bpp;
    Uint32 video_flags;

    /* Set default options and check command-line */
    w = 0;
    h = 0;
    desired_bpp = 0;
    video_flags = 0;

    while (1) {
	int c;
	static const struct option long_options[] =
	{
	    {"width", 1, 0, 'W'},
	    {"height", 1, 0, 'H'},
	    {"bpp", 1, 0, 'b'},
	    {"fps", 1, 0, 'f'},
	    {"rating", 1, 0, 'r'},
	    {"antialias", 0, 0, 'a'},
	    {"nogrid", 0, 0, 'G'},
	    {"nozoom", 0, 0, 'Z'},
	    {"norotate", 0, 0, 'R'},
	    {"fullscreen", 0, 0, 'F'},
	    {"resizable", 0, 0, 's'},
	    {"hwpalette", 0, 0, 'P'},
	    {"hwsurface", 0, 0, 'S'},
	    {"invulnerable", 0, 0, 'i'},
	    {"version", 0, 0, 'V'},
	    {"help", 0, 0, 'h'},
	    {0,0,0,0}
	};
	c = getopt_long(argc, argv, "W:H:b:f:r:I:aGZRFsPSiVh", long_options, NULL);
	if (c == -1)
	    break;
	switch (c)
	{
	    case 'W':
		w = atoi(optarg);
		break;
	    case 'H':
		h = atoi(optarg);
		break;
	    case 'b':
		desired_bpp = atoi(optarg);
		break;
	    case 'f':
		settings.fps = atoi(optarg);
		break;
	    case 'r':
		if (1.0 <= atof(optarg))
		    gameState.setRating(atof(optarg));
		else
		    printf("bad rating\n");
		break;
	    case 'F':
		video_flags |= SDL_FULLSCREEN;
		break;
	    case 's':
		video_flags |= SDL_RESIZABLE;
		break;
	    case 'P':
		video_flags |= SDL_HWPALETTE;
		break;
	    case 'S':
		video_flags |= SDL_HWSURFACE;
		break;
	    case 'Z':
		settings.zoomEnabled = false;
		break;
	    case 'R':
		settings.rotatingView = false;
		break;
	    case 'a':
		settings.useAA = true;
		break;
	    case 'G':
		settings.showGrid = false;
		break;
	    case 'i':
		settings.invuln = true;
		break;
	    case 'V':
		printf("%s\n", PACKAGE_STRING);
		exit(0);
	    case 'h':
		printf("Options:\n\t"
			"-W --width WIDTH\n\t-H --height HEIGHT\n\t-b --bpp BITS\n\t-f --fps FPS\n\t"
			"-F --fullscreen\n\t-S --resizable\n\t-P --hwpalette\n\t-S --hwsurface\n\t"
			"-Z --nozoom\n\t-R --norotate\n\t-G --nogrid\n\t-a --antialias\n\t"
			"-r --rating RATING\t\t1: harmless... 4: average... 9: elite\n\t"
			"-i --invulnerable\n\t-V --version\n\t-h --help\n");
		exit(0);
		break;

	    case '?':
		break;

	    default:
		printf("getopt returned unexpected character %c\n", c);
	}
    }

    if (gameState.rating == 0)
    {
	gameState.setRating(config.rating);
	config.shouldUpdateRating = true;
    }

    /* Force double buffering */
    video_flags |= SDL_DOUBLEBUF;

    /* Initialize SDL */
    if ( SDL_Init(SDL_INIT_EVERYTHING) < 0 ) {
	fprintf(stderr,
		"Couldn't initialize SDL: %s\n", SDL_GetError());
	exit(1);
    }
    atexit(SDL_Quit);			/* Clean up on exit */

    if (w == 0 || h == 0)
    {
	SDL_Rect** modes = SDL_ListModes(NULL, video_flags);
	if (modes == NULL)
	{
	    fprintf(stderr,
		    "SDL reports no modes available\n");
	    exit(1);
	}
	if (modes == (SDL_Rect**)(-1))
	{
	    // "All modes available"
	    w = 1024;
	    h = 768;
	}
	else
	{
	    // use the first (i.e. biggest) mode which is no bigger than
	    // 1024x768
	    SDL_Rect* bestMode;
	    do
	    {
		bestMode = *modes;
		if (bestMode->w <= 1024 && bestMode->h <= 768)
		    break;
	    } while (++modes);
	    w = bestMode->w;
	    h = bestMode->h;
	}
    }
	

    /* Initialize the display */
    screen = SDL_SetVideoMode(w, h, desired_bpp, video_flags);
    if ( screen == NULL ) {
	fprintf(stderr, "Couldn't set %dx%dx%d video mode: %s\n",
		w, h, desired_bpp, SDL_GetError());
	exit(1);
    }

    if (desired_bpp == 0 && screen->format->BitsPerPixel == 32)
    {
	/* There appears to be a bug in current (2.0.17) SDL_gfx relating to
	 * alpha blending in 32bpp mode. To work around this, for now we only
	 * use 32bpp if it was explicitly asked for.
	 */
	printf("Changing colour depth from 32 to 24; run with -b32 to force\n");

	screen = SDL_SetVideoMode(w, h, 24, video_flags);
	if ( screen == NULL ) {
	    fprintf(stderr, "Couldn't set %dx%dx%d video mode: %s\n",
		    w, h, 24, SDL_GetError());
	    exit(1);
	}
    }

    /* Show some info */
    printf("Set %dx%dx%d mode\n",
	    screen->w, screen->h, screen->format->BitsPerPixel);
    printf("Video surface located in %s memory.\n",
	    (screen->flags&SDL_HWSURFACE) ? "video" : "system");

    /* Check for double buffering */
    if ( screen->flags & SDL_DOUBLEBUF ) {
	printf("Double-buffering enabled - good!\n");
    }

    SDL_WM_SetCaption( "Kuklomenos", NULL );

    SDL_ShowCursor(SDL_DISABLE);

    srand(time(NULL));

    if ( !initFont() )
    {
	fprintf(stderr, "Failed to load font data file\n");
	exit(1);
    }

    screenGeom = ScreenGeom(w,h);

    // main loop
    Uint32 lastStateUpdate = 0;
    Uint32 ticksBefore, ticksAfter;
    Uint32 loopTicks = SDL_GetTicks();
    int timeTillNextFrame = 0;
    int delayTime, updateTime;
    float avFrameTime = 1000/settings.fps;
    int fpsRegulatorTenthTicks = 0;
    const int avFrames = 10; // number of frames to average over
    EventsReturn eventsReturn = ER_NONE;
    bool quit = false;
    bool ended = false;
    while ( !quit ) {
	while (timeTillNextFrame > 0)
	{
	    delayTime = std::min(timeTillNextFrame, MIN_STEP);
	    SDL_Delay(delayTime);
	    timeTillNextFrame -= delayTime;

	    eventsReturn = process_events(gameState, gameClock);

	    switch (eventsReturn)
	    {
		case ER_RESTART:
		    {
			GameState newGameState;
			newGameState.setRating(gameState.rating);
			gameState = newGameState;
		    }
		    ended = false;
		    overlayText.blank();
		    break;
		case ER_QUIT:
		    quit = true;
		default: ;
	    }

	    updateTime = SDL_GetTicks() - lastStateUpdate;
	    lastStateUpdate += updateTime;
	    if (!gameClock.paused)
	    {
		gameState.update(gameClock.scale(updateTime));
		gameClock.update(updateTime);
	    }
	}

	ticksBefore = SDL_GetTicks();
	if (!gameClock.paused)
	{
	    gameState.draw(screen);
	    drawInfo(screen, gameState, gameClock, 1000.0/avFrameTime);
	    overlayText.draw(screen);
	    SDL_Flip(screen);
	}
	ticksAfter = SDL_GetTicks();

	const int renderingTicks = ticksAfter - ticksBefore;

	// Add in a manual tweak to the delay, to ensure we approximate the
	// requested fps (where possible):
	if (10000/avFrameTime < settings.fps * 10 &&
		1000/settings.fps - renderingTicks + fpsRegulatorTenthTicks/10 > 1)
	    fpsRegulatorTenthTicks -= 1;
	else if (10000/avFrameTime > settings.fps * 10)
	    fpsRegulatorTenthTicks += 1;

	timeTillNextFrame =
	    std::max(1, (1000/settings.fps) - renderingTicks +
		    fpsRegulatorTenthTicks/10 + (rani(10) < fpsRegulatorTenthTicks%10));

	if (!ended && gameState.end)
	{
	    ended = true;
	    switch (gameState.end)
	    {
		case END_DEAD:
		case END_EXTRACTED:
		    snprintf(overlayText.text1, overlayText.text1Len, "Failure.");
		    overlayText.colour = 0xff4444ff;
		    break;
		case END_WIN:
		    snprintf(overlayText.text1, overlayText.text1Len, "Victory!");
		    overlayText.colour = 0x88ff00ff;
		    break;
		default: ;
	    }

	    if (config.shouldUpdateRating && !settings.invuln)
	    {
		int old = (int)(gameState.rating);

		if (gameState.end == END_DEAD ||
			gameState.end == END_EXTRACTED)
		{
		    gameState.rating -= 0.1;
		    if (gameState.rating < 1)
			gameState.rating = 1.0;

		}
		else if (gameState.end == END_WIN)
		{
		    gameState.rating += 0.4;
		    if (gameState.rating > 9)
			gameState.rating = 9.0;

		}

		if ((int)(gameState.rating) != old)
		    snprintf(overlayText.text2, overlayText.text2Len, "New rating: \"%s\".",
			    ratingString( (int)(gameState.rating) ));
	    }
	}

	const int loopTime = SDL_GetTicks() - loopTicks;
	avFrameTime += (loopTime-avFrameTime)/avFrames;
	loopTicks = SDL_GetTicks();
    }
    
    if (gameState.extracted > 0)
    {
	// game is forfeit - reduce rating
	gameState.rating -= 0.1;
	if (gameState.rating < 1)
	    gameState.rating = 1.0;
    }
    if (config.shouldUpdateRating)
    {
	config.rating = gameState.rating;
	config.write();
    }
    SDL_Quit();
}
